from __future__ import annotations

from shadowsocks import protocol_flag as flag
from shadowsocks.ciphers import SUPPORT_METHODS
from shadowsocks.mdb.models import User
from shadowsocks.metrics import (
    DECRYPT_DATA_TIME,
    ENCRYPT_DATA_TIME,
    NETWORK_TRANSMIT_BYTES,
)
from shadowsocks.utils import AutoResetBloomFilter


class CipherMan:
    bf = AutoResetBloomFilter()

    # TODO 流量、链接数限速

    def __init__(
        self,
        user_port=None,
        access_user: User = None,
        ts_protocol=flag.TRANSPORT_TCP,
        peername=None,
    ):
        self.user_port = user_port
        self.access_user = access_user
        self.ts_protocol = ts_protocol
        self.peername = peername

        self.cipher = None
        self._buffer = bytearray()

        if self.access_user:
            self.method = access_user.method
        else:
            self.method = (
                User.list_by_port(self.user_port).first().method
            )  # NOTE 所有的user用的加密方式必须是一种

        self.cipher_cls = SUPPORT_METHODS.get(self.method)
        if not self.cipher_cls:
            raise Exception(f"暂时不支持这种加密方式:{self.method}")
        if self.cipher_cls.AEAD_CIPHER and self.ts_protocol == flag.TRANSPORT_TCP:
            self._first_data_len = self.cipher_cls.tcp_first_data_len()
        else:
            self._first_data_len = 0

    @classmethod
    def get_cipher_by_port(cls, port, ts_protocol, peername) -> CipherMan:
        user_query = User.list_by_port(port)
        if user_query.count() == 1:
            access_user = user_query.first()
        else:
            access_user = None
        return cls(
            port, access_user=access_user, ts_protocol=ts_protocol, peername=peername
        )

    @ENCRYPT_DATA_TIME.time()
    def encrypt(self, data: bytes):
        self.record_user_traffic(0, len(data))

        if self.ts_protocol == flag.TRANSPORT_UDP:
            cipher = self.cipher_cls(self.access_user.password)
            return cipher.pack(data)

        if not self.cipher:
            self.cipher = self.cipher_cls(self.access_user.password)
        return self.cipher.encrypt(data)

    @DECRYPT_DATA_TIME.time()
    def decrypt(self, data: bytes):
        if (
            self.access_user is None
            and len(data) + len(self._buffer) < self._first_data_len
        ):
            self._buffer.extend(data)
            return

        if not self.access_user:
            self._buffer.extend(data)
            if self.ts_protocol == flag.TRANSPORT_TCP:
                first_data = self._buffer[: self._first_data_len]
            else:
                first_data = self._buffer
            salt = first_data[: self.cipher_cls.SALT_SIZE]
            if salt in self.bf:
                raise RuntimeError(
                    f"repeated salt founded!,peer:{self.peername},{salt}"
                )
            else:
                self.bf.add(salt)

            access_user = User.find_access_user(
                self.user_port,
                self.method,
                self.ts_protocol,
                first_data,
            )

            if not access_user:
                raise RuntimeError(
                    f"can not find enable access user: {self.user_port}-{self.ts_protocol}-{self.cipher_cls}"
                )
            if not access_user.enable:
                raise RuntimeError(f"access user not have traffic: {access_user}")
            self.access_user = access_user
            self.record_user_ip(self.peername)
            self.incr_user_tcp_num(1)
            data = bytes(self._buffer)
        if not self.cipher:
            self.cipher = self.cipher_cls(self.access_user.password)

        self.record_user_traffic(len(data), 0)
        if self.ts_protocol == flag.TRANSPORT_TCP:
            return self.cipher.decrypt(data)
        else:
            return self.cipher_cls(self.access_user.password).unpack(data)

    def incr_user_tcp_num(self, num: int):
        self.access_user and self.access_user.incr_tcp_conn_num(num)

    def record_user_ip(self, peername):
        self.access_user and self.access_user.record_ip(peername)

    def record_user_traffic(self, ut_data_len: int, dt_data_len: int):
        self.access_user and self.access_user.record_traffic(ut_data_len, dt_data_len)
        NETWORK_TRANSMIT_BYTES.inc(ut_data_len + dt_data_len)

    def close(self):
        self.incr_user_tcp_num(-1)



import abc
import hashlib
import os

import hkdf
from cryptography.hazmat.primitives.ciphers.aead import AESGCM, ChaCha20Poly1305


def evp_bytestokey(password: bytes, key_size: int):
    """make user password stronger
    openssl EVP_BytesToKey python implement
    doc: https://www.openssl.org/docs/manmaster/man3/EVP_BytesToKey.html
    """
    m = []
    for i in range(key_size // 16):
        if i > 0:
            data = m[i - 1] + password
        else:
            data = password
        md5 = hashlib.md5()
        md5.update(data)
        m.append(md5.digest())
    return b"".join(m)


class BaseCipher(metaclass=abc.ABCMeta):
    KEY_SIZE = -1
    AEAD_CIPHER = False

    def __init__(self, password: str):
        self.key = evp_bytestokey(password.encode(), self.KEY_SIZE)

    @abc.abstractmethod
    def new_cipher(self, *arg, **kwargs):
        return

    @abc.abstractmethod
    def encrypt(self, data: bytes):
        return

    @abc.abstractmethod
    def decrypt(self, data: bytes):
        return

    @abc.abstractmethod
    def unpack(self, data: bytes) -> bytes:
        return

    @abc.abstractmethod
    def pack(self, data: bytes) -> bytes:
        return


class BaseAEADCipher(BaseCipher):
    """DOC: https://shadowsocks.org/en/wiki/AEAD-Ciphers
    TCP: [encrypted payload length][length tag][encrypted payload][payload tag]
    UDP: [salt][encrypted payload][tag]
    """

    INFO = b"ss-subkey"
    PACKET_LIMIT = 16 * 1024 - 1
    SALT_SIZE = -1
    NONCE_SIZE = -1
    TAG_SIZE = -1
    AEAD_CIPHER = True

    def __init__(self, password: str):
        super().__init__(password)
        self._buffer = bytearray()
        self._payload_len = None
        self._subkey = None
        self._counter = 0
        self._cipher = None

    def _derive_subkey(self, salt: bytes):
        return hkdf.Hkdf(salt, self.key, hashlib.sha1).expand(self.INFO, self.KEY_SIZE)

    def _make_random_salt(self):
        return os.urandom(self.SALT_SIZE)

    def _encrypt(self, plaintext: bytes):
        if not self._cipher:
            self._cipher = self.new_cipher(self._subkey)
        return self._cipher.encrypt(self.nonce, plaintext, None)

    def _decrypt(self, ciphertext: bytes, tag: bytes):
        if not self._cipher:
            self._cipher = self.new_cipher(self._subkey)
        return self._cipher.decrypt(self.nonce, bytes(ciphertext + tag), None)

    @property
    def nonce(self):
        ret = self._counter.to_bytes(self.NONCE_SIZE, "little")
        self._counter += 1
        return ret

    def encrypt(self, data: bytes) -> bytes:
        ret = bytearray()
        if self._subkey is None:
            salt = self._make_random_salt()
            self._subkey = self._derive_subkey(salt)
            ret.extend(salt)
        for i in range(0, len(data), self.PACKET_LIMIT):
            buf = data[i : i + self.PACKET_LIMIT]
            #  len_chunk, len_tag  + body_chunk + body_tag
            ret.extend(self._encrypt(len(buf).to_bytes(2, "big")) + self._encrypt(buf))
        return bytes(ret)

    def decrypt(self, data: bytes) -> bytes:
        ret = bytearray()
        if self._subkey is None:
            salt, data = data[: self.SALT_SIZE], data[self.SALT_SIZE :]
            self._subkey = self._derive_subkey(salt)

        self._buffer.extend(data)

        while True:
            if not self._payload_len:
                # 从data里拿出payload_length
                if len(self._buffer) < 2 + self.TAG_SIZE:
                    break
                else:
                    self._payload_len = int.from_bytes(
                        self._decrypt(
                            self._buffer[:2], self._buffer[2 : 2 + self.TAG_SIZE]
                        ),
                        "big",
                    )
                    if self._payload_len > self.PACKET_LIMIT:
                        raise RuntimeError(f"payload_len too long {self.payload_len}")

                    del self._buffer[: 2 + self.TAG_SIZE]
            else:
                if len(self._buffer) < self._payload_len + self.TAG_SIZE:
                    break
                ret.extend(
                    self._decrypt(
                        self._buffer[: self._payload_len],
                        self._buffer[
                            self._payload_len : self._payload_len + self.TAG_SIZE
                        ],
                    )
                )
                del self._buffer[: self._payload_len + self.TAG_SIZE]
                self._payload_len = None

        return bytes(ret)

    def unpack(self, data: bytes) -> bytes:
        """解包udp"""
        ret = bytearray()
        data_len = len(data)
        salt, data, tag = (
            data[: self.SALT_SIZE],
            data[self.SALT_SIZE : data_len - self.TAG_SIZE],
            data[data_len - self.TAG_SIZE :],
        )
        self._subkey = self._derive_subkey(salt)
        ret.extend(self._decrypt(data, tag))
        return bytes(ret)

    def pack(self, data: bytes) -> bytes:
        """压udp包"""
        ret = bytearray()
        salt = self._make_random_salt()
        self._subkey = self._derive_subkey(salt)
        ret.extend(salt)
        ret.extend(self._encrypt(data))
        return bytes(ret)

    @classmethod
    def tcp_first_data_len(cls):
        return cls.SALT_SIZE + 2 + cls.TAG_SIZE


class NONE(BaseCipher):
    def new_cipher(self, key: bytes, iv: bytes):
        return

    def encrypt(self, data: bytes):
        return data

    def decrypt(self, data: bytes):
        return data

    def unpack(self, data: bytes) -> bytes:
        return data

    def pack(self, data: bytes) -> bytes:
        return data


class CHACHA20IETFPOLY1305(BaseAEADCipher):
    KEY_SIZE = 32
    SALT_SIZE = 32
    NONCE_SIZE = 12
    TAG_SIZE = 16

    def new_cipher(self, subkey: bytes):
        return ChaCha20Poly1305(key=subkey)


class AES128GCM(BaseAEADCipher):
    KEY_SIZE = 16
    SALT_SIZE = 16
    NONCE_SIZE = 12
    TAG_SIZE = 16

    def new_cipher(self, subkey: bytes):
        return AESGCM(subkey)


class AES256GCM(AES128GCM):
    KEY_SIZE = 32
    SALT_SIZE = 32
    NONCE_SIZE = 12
    TAG_SIZE = 16


# NOTE 目前提供所有AEAD的加密方式，流式加密只提供None（不加密）
# 但是所有流式加密的方式都不推荐使用了，生产环境请一律使用AEAD加密
SUPPORT_METHODS = {
    "none": NONE,
    "aes-128-gcm": AES128GCM,
    "aes-256-gcm": AES256GCM,
    "chacha20-ietf-poly1305": CHACHA20IETFPOLY1305,
}



from __future__ import annotations

import logging

import peewee as pw
from cryptography.exceptions import InvalidTag

from shadowsocks import protocol_flag as flag
from shadowsocks.ciphers import SUPPORT_METHODS
from shadowsocks.mdb import BaseModel, IPSetField, db
from shadowsocks.metrics import FIND_ACCESS_USER_TIME


class User(BaseModel):

    __attr_protected__ = {"user_id"}
    __attr_accessible__ = {"port", "method", "password", "enable", "speed_limit"}

    user_id = pw.IntegerField(primary_key=True, unique=True)
    port = pw.IntegerField(index=True)
    method = pw.CharField()
    password = pw.CharField(unique=True)
    enable = pw.BooleanField(default=True)
    speed_limit = pw.IntegerField(default=0)
    access_order = pw.BigIntegerField(
        index=True, default=0
    )  # NOTE find_access_user order
    need_sync = pw.BooleanField(default=False, index=True)
    # metrics field
    ip_list = IPSetField(default=set())
    tcp_conn_num = pw.IntegerField(default=0)
    upload_traffic = pw.BigIntegerField(default=0)
    download_traffic = pw.BigIntegerField(default=0)

    @classmethod
    def _create_or_update_user_from_data(cls, data):
        user_id = data.pop("user_id")
        user, created = cls.get_or_create(user_id=user_id, defaults=data)
        if not created:
            user.update_from_dict(data)
            user.save()
        logging.debug(f"正在创建/更新用户:{user}的数据")
        return user

    @classmethod
    def list_by_port(cls, port):
        fields = [
            cls.user_id,
            cls.method,
            cls.password,
            cls.enable,
            cls.ip_list,
            cls.access_order,
        ]
        return (
            cls.select(*fields)
            .where(cls.port == port)
            .order_by(cls.access_order.desc())
        )

    @classmethod
    @db.atomic("EXCLUSIVE")
    def create_or_update_by_user_data_list(cls, user_data_list):
        user_ids = []
        for user_data in user_data_list:
            user_ids.append(user_data["user_id"])
            cls._create_or_update_user_from_data(user_data)
        cnt = cls.delete().where(cls.user_id.not_in(user_ids)).execute()
        if cnt:
            logging.info(f"delete out of traffic user cnt: {cnt}")

    @db.atomic("EXCLUSIVE")
    def record_ip(self, peername):
        if not peername:
            return
        self.ip_list.add(peername[0])
        User.update(ip_list=self.ip_list, need_sync=True).where(
            User.user_id == self.user_id
        ).execute()

    @db.atomic("EXCLUSIVE")
    def record_traffic(self, used_u, used_d):
        User.update(
            download_traffic=User.download_traffic + used_d,
            upload_traffic=User.upload_traffic + used_u,
            need_sync=True,
        ).where(User.user_id == self.user_id).execute()

    @db.atomic("EXCLUSIVE")
    def incr_tcp_conn_num(self, num):
        User.update(tcp_conn_num=User.tcp_conn_num + num, need_sync=True).where(
            User.user_id == self.user_id
        ).execute()

    @classmethod
    @FIND_ACCESS_USER_TIME.time()
    def find_access_user(cls, port, method, ts_protocol, first_data) -> User:
        """先从访问的cache里寻找，找不到在去db里"""
        cipher_cls = SUPPORT_METHODS[method]
        access_user = None
        for user in cls.list_by_port(port).iterator():
            try:
                cipher = cipher_cls(user.password)
                if ts_protocol == flag.TRANSPORT_TCP:
                    cipher.decrypt(first_data)
                else:
                    cipher.unpack(first_data)
                access_user = user
                break
            except InvalidTag:
                pass
        if access_user:
            # NOTE 记下成功访问的用户，下次优先找到他
            access_user.access_order += 1
            access_user.save()
        return access_user



import json
from typing import Set

import peewee as pw
from playhouse import shortcuts

db = pw.SqliteDatabase(":memory:")


class BaseModel(pw.Model):
    __attr_protected__ = set()
    __attr_accessible__ = set()

    class Meta:
        database = db

    @classmethod
    def _filter_attrs(cls, attrs, use_whitelist=True):
        if use_whitelist:
            whitelist = cls.__attr_accessible__ - cls.__attr_protected__
            return {k: v for k, v in attrs.items() if k in whitelist}
        else:
            blacklist = cls.__attr_protected__ - cls.__attr_accessible__
            return {k: v for k, v in attrs.items() if k not in blacklist}

    @classmethod
    def get_or_create(cls, **kw):
        if "defaults" in kw:
            kw["defaults"] = cls._filter_attrs(kw.pop("defaults"))
        return super().get_or_create(**kw)

    def update_from_dict(self, data, ignore_unknown=False, use_whitelist=False):
        """注意值是没有写入数据库的, 需要显式 save"""
        cls = type(self)
        clean_data = cls._filter_attrs(data)
        return shortcuts.update_model_from_dict(self, clean_data, ignore_unknown)

    def to_dict(self, **kw):
        return shortcuts.model_to_dict(self, **kw)


class IPSetField(pw.CharField):
    def db_value(self, value) -> str:
        if type(value) is not set:
            value = []
        data = json.dumps(list(value))
        if len(data) > self.max_length:
            raise ValueError("Data too long for field {}.".format(self.name))
        return data

    def python_value(self, value) -> Set[str]:
        if value is None:
            return value
        l = json.loads(value)
        return set(l)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import aioshadowsocks_pb2 as aioshadowsocks__pb2


class ssStub(object):
    """service"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateUser = channel.unary_unary(
            "/aioshadowsocks.ss/CreateUser",
            request_serializer=aioshadowsocks__pb2.UserReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.User.FromString,
        )
        self.UpdateUser = channel.unary_unary(
            "/aioshadowsocks.ss/UpdateUser",
            request_serializer=aioshadowsocks__pb2.UserReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.User.FromString,
        )
        self.GetUser = channel.unary_unary(
            "/aioshadowsocks.ss/GetUser",
            request_serializer=aioshadowsocks__pb2.UserIdReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.User.FromString,
        )
        self.DeleteUser = channel.unary_unary(
            "/aioshadowsocks.ss/DeleteUser",
            request_serializer=aioshadowsocks__pb2.UserIdReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.Empty.FromString,
        )
        self.ListUser = channel.unary_unary(
            "/aioshadowsocks.ss/ListUser",
            request_serializer=aioshadowsocks__pb2.UserReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.UserList.FromString,
        )
        self.HealthCheck = channel.unary_stream(
            "/aioshadowsocks.ss/HealthCheck",
            request_serializer=aioshadowsocks__pb2.HealthCheckReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.HealthCheckRes.FromString,
        )
        self.FindAccessUser = channel.unary_unary(
            "/aioshadowsocks.ss/FindAccessUser",
            request_serializer=aioshadowsocks__pb2.FindAccessUserReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.User.FromString,
        )
        self.DecryptData = channel.unary_unary(
            "/aioshadowsocks.ss/DecryptData",
            request_serializer=aioshadowsocks__pb2.DecryptDataReq.SerializeToString,
            response_deserializer=aioshadowsocks__pb2.DecryptDataRes.FromString,
        )


class ssServicer(object):
    """service"""

    def CreateUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def HealthCheck(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def FindAccessUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DecryptData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ssServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "CreateUser": grpc.unary_unary_rpc_method_handler(
            servicer.CreateUser,
            request_deserializer=aioshadowsocks__pb2.UserReq.FromString,
            response_serializer=aioshadowsocks__pb2.User.SerializeToString,
        ),
        "UpdateUser": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateUser,
            request_deserializer=aioshadowsocks__pb2.UserReq.FromString,
            response_serializer=aioshadowsocks__pb2.User.SerializeToString,
        ),
        "GetUser": grpc.unary_unary_rpc_method_handler(
            servicer.GetUser,
            request_deserializer=aioshadowsocks__pb2.UserIdReq.FromString,
            response_serializer=aioshadowsocks__pb2.User.SerializeToString,
        ),
        "DeleteUser": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteUser,
            request_deserializer=aioshadowsocks__pb2.UserIdReq.FromString,
            response_serializer=aioshadowsocks__pb2.Empty.SerializeToString,
        ),
        "ListUser": grpc.unary_unary_rpc_method_handler(
            servicer.ListUser,
            request_deserializer=aioshadowsocks__pb2.UserReq.FromString,
            response_serializer=aioshadowsocks__pb2.UserList.SerializeToString,
        ),
        "HealthCheck": grpc.unary_stream_rpc_method_handler(
            servicer.HealthCheck,
            request_deserializer=aioshadowsocks__pb2.HealthCheckReq.FromString,
            response_serializer=aioshadowsocks__pb2.HealthCheckRes.SerializeToString,
        ),
        "FindAccessUser": grpc.unary_unary_rpc_method_handler(
            servicer.FindAccessUser,
            request_deserializer=aioshadowsocks__pb2.FindAccessUserReq.FromString,
            response_serializer=aioshadowsocks__pb2.User.SerializeToString,
        ),
        "DecryptData": grpc.unary_unary_rpc_method_handler(
            servicer.DecryptData,
            request_deserializer=aioshadowsocks__pb2.DecryptDataReq.FromString,
            response_serializer=aioshadowsocks__pb2.DecryptDataRes.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "aioshadowsocks.ss", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ss(object):
    """service"""

    @staticmethod
    def CreateUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/CreateUser",
            aioshadowsocks__pb2.UserReq.SerializeToString,
            aioshadowsocks__pb2.User.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/UpdateUser",
            aioshadowsocks__pb2.UserReq.SerializeToString,
            aioshadowsocks__pb2.User.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/GetUser",
            aioshadowsocks__pb2.UserIdReq.SerializeToString,
            aioshadowsocks__pb2.User.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/DeleteUser",
            aioshadowsocks__pb2.UserIdReq.SerializeToString,
            aioshadowsocks__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/ListUser",
            aioshadowsocks__pb2.UserReq.SerializeToString,
            aioshadowsocks__pb2.UserList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def HealthCheck(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/aioshadowsocks.ss/HealthCheck",
            aioshadowsocks__pb2.HealthCheckReq.SerializeToString,
            aioshadowsocks__pb2.HealthCheckRes.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def FindAccessUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/FindAccessUser",
            aioshadowsocks__pb2.FindAccessUserReq.SerializeToString,
            aioshadowsocks__pb2.User.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DecryptData(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/aioshadowsocks.ss/DecryptData",
            aioshadowsocks__pb2.DecryptDataReq.SerializeToString,
            aioshadowsocks__pb2.DecryptDataRes.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from time import time


class TrafficRateLimit:
    KB = 1024
    MEGABIT = KB * 125

    def __init__(self, capacity, rate=None):
        """
        令牌桶法按照带宽限速限速
        capacity: 带宽
        rate: 速率 capacity/second
        """
        self.capacity = float(capacity)
        if not rate:
            self.rate = self.capacity
        else:
            self.rate = rate

        self._remain_traffic = self.capacity
        self._last_time = time()
        self._cur_rate = 0

    def consume(self, traffic_lens):
        time_delta = time() - self._last_time
        self.fill(time_delta)

        # NOTE _remain_traffic 可以为负数
        self._remain_traffic -= traffic_lens
        self._cur_rate = traffic_lens / time_delta

    def fill(self, time_delta=None):
        if not time_delta:
            time_delta = time() - self._last_time
        self._last_time += time_delta

        if self._remain_traffic < 0:
            # NOTE 给超速的用户补上
            increment = time_delta * self.rate
            self._remain_traffic += increment
            return

        if time_delta > 1:
            self._remain_traffic = self.capacity
        else:
            increment = time_delta * self.rate
            self._remain_traffic = min(increment + self._remain_traffic, self.capacity)

    @property
    def cur_rate(self):
        return f"now rate is: {round(self._cur_rate / self.MEGABIT, 1)} Mbps"

    @property
    def limited(self):
        if self.capacity == float(0):
            return False
        self.fill()
        return self._remain_traffic < 0

    def get_sleep_time(self):
        if self._remain_traffic > 0:
            return 0
        else:
            return abs(self._remain_traffic) / self.rate


class TcpConnRateLimit:
    def __init__(self, capacity):
        self.capacity = capacity
        self.tcp_conn_num = 0

    def incr_tcp_conn_num(self, num):
        self.tcp_conn_num += num

    @property
    def limited(self):
        return self.tcp_conn_num > self.capacity



import asyncio
import logging
import socket
import struct

from shadowsocks import protocol_flag as flag
from shadowsocks.cipherman import CipherMan
from shadowsocks.metrics import ACTIVE_CONNECTION_COUNT, CONNECTION_MADE_COUNT
from shadowsocks.utils import parse_header


class LocalHandler:
    """
    事件循环一共处理五个状态

    STAGE_INIT  初始状态 socket5握手
    STAGE_CONNECT 连接建立阶段 从本地获取addr 进行dns解析
    STAGE_STREAM 建立管道(pipe) 进行socket5传输
    STAGE_DESTROY 结束连接状态
    STAGE_ERROR 异常状态
    """

    STAGE_INIT = 0
    STAGE_CONNECT = 1
    STAGE_STREAM = 2
    STAGE_DESTROY = -1
    STAGE_ERROR = 255

    def __init__(self, port):
        super().__init__()

        self.port = port
        self.cipher = None

        self._stage = None
        self._peername = None
        self._remote = None
        self._transport = None
        self._transport_protocol = None
        self._transport_protocol_human = None
        self._is_closing = False
        self._connect_buffer = bytearray()

    def _init_transport(self, transport: asyncio.Transport, peername, protocol):
        self._stage = self.STAGE_INIT
        self._transport = transport
        self._peername = peername
        self._transport_protocol = protocol
        if protocol == flag.TRANSPORT_TCP:
            self._transport_protocol_human = "tcp"
        else:
            self._transport_protocol_human = "udp"

    def close(self):
        self._stage = self.STAGE_DESTROY
        if self._is_closing:
            return
        self._is_closing = True
        ACTIVE_CONNECTION_COUNT.inc(-1)

        if self._transport_protocol == flag.TRANSPORT_TCP:
            self._transport and self._transport.close()
            self.cipher and self.cipher.close()
        self._remote and self._remote.close()

    def write(self, data):
        if self._transport_protocol == flag.TRANSPORT_TCP:
            if self._transport.is_closing():
                return
            self._transport.write(data)
        else:
            self._transport.sendto(data, self._peername)

    def handle_connection_made(self, transport_protocol, transport, peername):
        self._init_transport(transport, peername, transport_protocol)

    def handle_eof_received(self):
        self.close()

    def handle_connection_lost(self, exc):
        self.close()

    def handle_data_received(self, data):

        if not self.cipher:
            self.cipher = CipherMan.get_cipher_by_port(
                self.port, self._transport_protocol, self._peername
            )

        try:
            data = self.cipher.decrypt(data)
        except Exception as e:
            logging.warning(
                f"decrypt data error:{e} remote:{self._peername},type:{self._transport_protocol_human} closing..."
            )
            self.close()
            return

        if not data:
            return

        if self._stage == self.STAGE_INIT:
            asyncio.create_task(self._handle_stage_init(data))
        elif self._stage == self.STAGE_CONNECT:
            self._handle_stage_connect(data)
        elif self._stage == self.STAGE_STREAM:
            self._handle_stage_stream(data)
        elif self._stage == self.STAGE_ERROR:
            self.close()
        elif self._stage == self.STAGE_DESTROY:
            self.close()
        else:
            logging.warning(f"unknown stage:{self._stage}")

    async def _handle_stage_init(self, data):
        atype, dst_addr, dst_port, header_length = parse_header(data)
        if not all([atype, dst_addr, dst_port, header_length]):
            logging.warning(
                f"parse_header_error atype: {flag.get_atype_for_human(atype)} port: {self.port}"
            )
            self.close()
            return
        else:
            logging.info(
                "parse_header_success atype: {} {} from: {} dst: {}:{}".format(
                    self._transport_protocol_human,
                    flag.get_atype_for_human(atype),
                    self._peername[0],
                    dst_addr,
                    dst_port,
                )
            )
            payload = data[header_length:]

        loop = asyncio.get_running_loop()
        if self._transport_protocol == flag.TRANSPORT_TCP:
            self._stage = self.STAGE_CONNECT
            self._handle_stage_connect(payload)
            try:
                task = loop.create_connection(
                    lambda: RemoteTCP(self), dst_addr, dst_port
                )
                _, remote_tcp = await asyncio.wait_for(task, 5)
            except Exception as e:
                self._stage = self.STAGE_ERROR
                self.close()
                logging.warning(
                    f"connection_failed, {type(e)} e: {dst_addr}:{dst_port}"
                )
            else:
                self._remote = remote_tcp
        else:
            try:
                task = loop.create_datagram_endpoint(
                    lambda: RemoteUDP(dst_addr, dst_port, payload, self),
                    remote_addr=(dst_addr, dst_port),
                )
                await asyncio.wait_for(task, 5)
            except Exception as e:
                self._stage = self.STAGE_ERROR
                self.close()
                logging.warning(
                    f"connection_failed, {type(e)} e: {dst_addr}:{dst_port}"
                )

    def _handle_stage_connect(self, data):
        # 在握手之后，会耗费一定时间来来和remote建立连接,但是ss-client并不会等这个时间
        if not self._remote or self._remote.ready == False:
            self._connect_buffer.extend(data)
        else:
            self._stage = self.STAGE_STREAM
            self._handle_stage_stream(data)

    def _handle_stage_stream(self, data):
        self._remote.write(data)


class LocalTCP(asyncio.Protocol):
    """
    Local Tcp Factory
    """

    def __init__(self, port):
        self.port = port
        self._handler = None

    def _init_handler(self):
        self._handler = LocalHandler(self.port)

    def __call__(self):
        local = LocalTCP(self.port)
        local._init_handler()
        return local

    def pause_writing(self):
        self._handler._remote._transport.pause_reading()

    def resume_writing(self):
        self._handler._remote._transport.resume_reading()

    def connection_made(self, transport):
        self._transport = transport
        peername = self._transport.get_extra_info("peername")
        self._handler.handle_connection_made(flag.TRANSPORT_TCP, transport, peername)
        CONNECTION_MADE_COUNT.inc()
        ACTIVE_CONNECTION_COUNT.inc()

    def data_received(self, data):
        self._handler.handle_data_received(data)

    def eof_received(self):
        self._handler.handle_eof_received()

    def connection_lost(self, exc):
        self._handler.handle_connection_lost(exc)


class RemoteTCP(asyncio.Protocol):
    def __init__(self, local_handler):
        super().__init__()

        self.local = local_handler
        self.peername = None
        self._transport = None
        self.ready = False

        self._is_closing = False

    def write(self, data):
        if not self._transport.is_closing():
            self._transport.write(data)

    def close(self):
        if self._is_closing:
            return
        self._is_closing = True
        ACTIVE_CONNECTION_COUNT.inc(-1)

        self._transport and self._transport.close()
        self.local.close()

    def connection_made(self, transport: asyncio.Transport):
        self._transport = transport
        self.peername = self._transport.get_extra_info("peername")
        self.cipher = CipherMan(
            access_user=self.local.cipher.access_user, peername=self.peername
        )
        transport.write(self.local._connect_buffer)
        self.ready = True
        CONNECTION_MADE_COUNT.inc()
        ACTIVE_CONNECTION_COUNT.inc()

    def data_received(self, data):
        self.local.write(self.cipher.encrypt(data))

    def pause_reading(self):
        self.local._transport.pause_reading()

    def resume_reading(self):
        self.local._transport.resume_reading()

    def eof_received(self):
        self.close()

    def connection_lost(self, exc):
        self.close()


class LocalUDP(asyncio.DatagramProtocol):
    """
    Local Udp Factory
    """

    def __init__(self, port):
        self.port = port
        self._protocols = {}
        self._transport = None

    def __call__(self):
        local = LocalUDP(self.port)
        return local

    def connection_made(self, transport):
        self._transport = transport

    def datagram_received(self, data, peername):
        if peername in self._protocols:
            handler = self._protocols[peername]
        else:
            handler = LocalHandler(self.port)
            self._protocols[peername] = handler
            handler.handle_connection_made(
                flag.TRANSPORT_UDP, self._transport, peername
            )
        handler.handle_data_received(data)

    def error_received(self, exc):
        # TODO clean udp conn
        pass


class RemoteUDP(asyncio.DatagramProtocol):
    def __init__(self, addr, port, data, local_hander):
        super().__init__()
        self.data = data
        self.local = local_hander
        self.peername = None
        self._transport = None
        self.cipher = CipherMan(
            access_user=self.local.cipher.access_user, ts_protocol=flag.TRANSPORT_UDP
        )
        self._is_closing = False

    def write(self, data):
        self._transport and not self._transport.is_closing() and self._transport.sendto(
            data
        )

    def close(self):
        if self._is_closing:
            return
        self._is_closing = True

        self._transport and self._transport.close()
        del self.local

    def connection_made(self, transport):
        self._transport = transport
        self.peername = self._transport.get_extra_info("peername")
        self.write(self.data)

    def datagram_received(self, data, peername, *arg):

        assert self.peername == peername
        # 源地址和端口
        bind_addr = peername[0]
        bind_port = peername[1]
        if "." in bind_addr:
            addr = socket.inet_pton(socket.AF_INET, bind_addr)
        elif ":" in bind_addr:
            addr = socket.inet_pton(socket.AF_INET6, bind_addr)
        else:
            raise Exception("add not valid")
        port = struct.pack("!H", bind_port)
        # 构造返回的报文结构
        data = b"\x01" + addr + port + data
        data = self.cipher.encrypt(data)
        self.local.write(data)

    def error_received(self, exc):
        self.close()

    def connection_lost(self, exc):
        self.close()



import logging
import socket
import struct

from bloom_filter import BloomFilter

from shadowsocks import protocol_flag as flag


def parse_header(data):
    # shadowsocks protocol https://shadowsocks.org/en/spec/Protocol.html
    atype, dst_addr, dst_port, header_length = data[0], None, None, 0
    if atype == flag.ATYPE_IPV4:
        if len(data) >= 7:
            dst_addr = socket.inet_ntop(socket.AF_INET, data[1:5])
            dst_port = struct.unpack("!H", data[5:7])[0]
            header_length = 7
        else:
            logging.warning("header is too short")
    elif atype == flag.ATYPE_IPV6:
        if len(data) >= 19:
            dst_addr = socket.inet_ntop(socket.AF_INET6, data[1:17])
            dst_port = struct.unpack("!H", data[17:19])[0]
            header_length = 19
        else:
            logging.warning("header is too short")
    elif atype == flag.ATYPE_DOMAINNAME:
        if len(data) > 2:
            addrlen = data[1]
            if len(data) >= 4 + addrlen:
                domain = data[2 : 2 + addrlen]
                dst_addr = domain.decode()
                dst_port = struct.unpack("!H", data[2 + addrlen : addrlen + 4])[0]
                header_length = 4 + addrlen
            else:
                logging.warning("header is too short")
        else:
            logging.warning("header is too short")
    else:
        logging.warning(f"unknown atype: {atype}")
    return atype, dst_addr, dst_port, header_length


class AutoResetBloomFilter:

    MAX_ELEMENTS = 10 ** 6
    ERROR_RATE = 10 ** -6

    def new_bf(self):
        self.size = self.MAX_ELEMENTS
        return BloomFilter(max_elements=self.MAX_ELEMENTS, error_rate=self.ERROR_RATE)

    def __init__(self):
        self.bf = self.new_bf()

    def add(self, v):
        if self.size <= 0:
            logging.warning("bloom filter reset")
            self.bf = self.new_bf()
        self.bf.add(v)
        self.size -= 1

    def __contains__(self, key):
        return key in self.bf



import asyncio
import inspect
import logging
import os
import signal

import sentry_sdk
from aiohttp import web
from grpclib.events import RecvRequest, listen
from grpclib.server import Server
from prometheus_async import aio
from sentry_sdk.integrations.aiohttp import AioHttpIntegration

from shadowsocks.mdb import BaseModel, models
from shadowsocks.proxyman import ProxyMan
from shadowsocks.services import AioShadowsocksServicer


async def logging_grpc_request(event: RecvRequest) -> None:
    logging.info(f"{event.method_name} called!")


class App:
    def __init__(self) -> None:
        self._prepared = False

    def _init_config(self):
        self.config = {
            "LISTEN_HOST": os.getenv("SS_LISTEN_HOST", "0.0.0.0"),
            "GRPC_HOST": os.getenv("SS_GRPC_HOST", "127.0.0.1"),
            "GRPC_PORT": os.getenv("SS_GRPC_PORT", "5000"),
            "SENTRY_DSN": os.getenv("SS_SENTRY_DSN"),
            "API_ENDPOINT": os.getenv("SS_API_ENDPOINT"),
            "LOG_LEVEL": os.getenv("SS_LOG_LEVEL", "info"),
            "SYNC_TIME": int(os.getenv("SS_SYNC_TIME", 60)),
            "STREAM_DNS_SERVER": os.getenv("SS_STREAM_DNS_SERVER"),
            "METRICS_PORT": os.getenv("SS_METRICS_PORT"),
            "TIME_OUT_LIMIT": int(os.getenv("SS_TIME_OUT_LIMIT", 60)),
            "USER_TCP_CONN_LIMIT": int(os.getenv("SS_TCP_CONN_LIMIT", 60)),
        }

        self.grpc_host = self.config["GRPC_HOST"]
        self.grpc_port = self.config["GRPC_PORT"]
        self.log_level = self.config["LOG_LEVEL"]
        self.sync_time = self.config["SYNC_TIME"]
        self.sentry_dsn = self.config["SENTRY_DSN"]
        self.listen_host = self.config["LISTEN_HOST"]
        self.api_endpoint = self.config["API_ENDPOINT"]
        self.timeout_limit = self.config["TIME_OUT_LIMIT"]
        self.stream_dns_server = self.config["STREAM_DNS_SERVER"]
        self.user_tcp_conn_limit = self.config["USER_TCP_CONN_LIMIT"]
        self.metrics_port = self.config["METRICS_PORT"]

        self.use_sentry = True if self.sentry_dsn else False
        self.use_json = False if self.api_endpoint else True
        self.metrics_server = None
        self.grpc_server = None

    def _init_logger(self):
        """
        basic log config
        """
        log_levels = {
            "CRITICAL": 50,
            "ERROR": 40,
            "WARNING": 30,
            "INFO": 20,
            "DEBUG": 10,
        }
        level = log_levels[self.log_level.upper()]
        logging.basicConfig(
            format="[%(levelname)s]%(asctime)s %(funcName)s line:%(lineno)d %(message)s",
            level=level,
        )

    def _init_memory_db(self):

        for _, model in inspect.getmembers(models, inspect.isclass):
            if issubclass(model, BaseModel) and model != BaseModel:
                model.create_table()
                logging.info(f"正在创建{model}内存数据库")

    def _init_sentry(self):
        if not self.use_sentry:
            return
        sentry_sdk.init(dsn=self.sentry_dsn, integrations=[AioHttpIntegration()])
        logging.info("Init Sentry Client...")

    def _prepare(self):
        if self._prepared:
            return
        self.loop = asyncio.get_event_loop()
        self._init_config()
        self._init_logger()
        self._init_memory_db()
        self._init_sentry()
        self.loop.add_signal_handler(signal.SIGTERM, self._shutdown)
        self.proxyman = ProxyMan(
            self.use_json, self.sync_time, self.listen_host, self.api_endpoint
        )
        self._prepared = True

    def _shutdown(self):
        logging.info("正在关闭所有ss server")
        self.proxyman.close_server()
        if self.grpc_server:
            self.grpc_server.close()
            logging.info(f"grpc server closed!")
        if self.metrics_server:
            self.loop.create_task(self.metrics_server.stop())
            logging.info(f"metrics server closed!")
        self.loop.stop()

    def _run_loop(self):

        try:
            self.loop.run_forever()
        except KeyboardInterrupt:
            self._shutdown()

    async def _start_grpc_server(self):

        self.grpc_server = Server([AioShadowsocksServicer()], loop=self.loop)
        listen(self.grpc_server, RecvRequest, logging_grpc_request)
        await self.grpc_server.start(self.grpc_host, self.grpc_port)
        logging.info(f"Start grpc Server on {self.grpc_host}:{self.grpc_port}")

    async def _start_metrics_server(self):
        app = web.Application()
        app.router.add_get("/metrics", aio.web.server_stats)
        runner = web.AppRunner(app)
        await runner.setup()
        self.metrics_server = web.TCPSite(runner, "0.0.0.0", self.metrics_port)
        await self.metrics_server.start()
        logging.info(
            f"Start Metrics Server At: http://0.0.0.0:{self.metrics_port}/metrics"
        )

    def run_ss_server(self):
        self._prepare()
        self.loop.create_task(self.proxyman.start_and_check_ss_server())
        if self.metrics_port:
            self.loop.create_task(self._start_metrics_server())
        self._run_loop()

    def run_grpc_server(self):
        self._prepare()

        if self.grpc_host and self.grpc_port:
            self.loop.create_task(self._start_grpc_server())
        else:
            raise Exception("grpc server not config")

        self._run_loop()



"""
shadowsocks 报文结构

+----+-----+-------+------+----------+----------+
|VER | CMD |  RSV  | ATYP | DST.ADDR | DST.PORT |
+----+-----+-------+------+----------+----------+
| 1  |  1  |   1   |  1   | Variable |    2     |
+----+-----+-------+------+----------+----------+
"""

# CMDS
TRANSPORT_TCP = 0x01
TRANSPORT_UDP = 0x02


# ATYPS
ATYPE_IPV4 = 0x01
ATYPE_IPV6 = 0x04
ATYPE_DOMAINNAME = 0x03

ATYPE_MAP = {ATYPE_IPV4: "ipv4", ATYPE_IPV6: "ipv6", ATYPE_DOMAINNAME: "domain"}


def get_atype_for_human(atype):
    if atype in ATYPE_MAP:
        return ATYPE_MAP[atype]
    return "unknown"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aioshadowsocks.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="aioshadowsocks.proto",
    package="aioshadowsocks",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x14\x61ioshadowsocks.proto\x12\x0e\x61ioshadowsocks"\x1c\n\tUserIdReq\x12\x0f\n\x07user_id\x18\x01 \x01(\x05"\x17\n\x07PortReq\x12\x0c\n\x04port\x18\x01 \x01(\x05"p\n\x07UserReq\x12\x0f\n\x07user_id\x18\x01 \x01(\x05\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x0e\n\x06method\x18\x03 \x01(\t\x12\x10\n\x08password\x18\x04 \x01(\t\x12\x0e\n\x06\x65nable\x18\x05 \x01(\x08\x12\x14\n\x0ctcp_conn_num\x18\x06 \x01(\x05"\x1d\n\x0eHealthCheckReq\x12\x0b\n\x03url\x18\x01 \x01(\t"7\n\x0eHealthCheckRes\x12\x13\n\x0bstatus_code\x18\x01 \x01(\t\x12\x10\n\x08\x64uration\x18\x02 \x01(\x05"\x07\n\x05\x45mpty"\xee\x01\n\x04User\x12\x0f\n\x07user_id\x18\x01 \x01(\x05\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x0e\n\x06method\x18\x03 \x01(\t\x12\x10\n\x08password\x18\x04 \x01(\t\x12\x0e\n\x06\x65nable\x18\x05 \x01(\x08\x12\x13\n\x0bspeed_limit\x18\x06 \x01(\x05\x12\x14\n\x0c\x61\x63\x63\x65ss_order\x18\x07 \x01(\x05\x12\x11\n\tneed_sync\x18\x08 \x01(\x08\x12\x0f\n\x07ip_list\x18\t \x03(\t\x12\x14\n\x0ctcp_conn_num\x18\n \x01(\x05\x12\x16\n\x0eupload_traffic\x18\x0b \x01(\x03\x12\x18\n\x10\x64ownload_traffic\x18\x0c \x01(\x03".\n\x08UserList\x12"\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.aioshadowsocks.User"T\n\x11\x46indAccessUserReq\x12\x0c\n\x04port\x18\x01 \x01(\x05\x12\x0e\n\x06method\x18\x02 \x01(\t\x12\x13\n\x0bts_protocol\x18\x03 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c"_\n\x0e\x44\x65\x63ryptDataReq\x12\x0f\n\x07user_id\x18\x01 \x01(\x05\x12\x0c\n\x04uuid\x18\x02 \x01(\t\x12\x10\n\x08password\x18\x03 \x01(\t\x12\x0e\n\x06method\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c"\x1e\n\x0e\x44\x65\x63ryptDataRes\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x32\xb4\x04\n\x02ss\x12=\n\nCreateUser\x12\x17.aioshadowsocks.UserReq\x1a\x14.aioshadowsocks.User"\x00\x12=\n\nUpdateUser\x12\x17.aioshadowsocks.UserReq\x1a\x14.aioshadowsocks.User"\x00\x12<\n\x07GetUser\x12\x19.aioshadowsocks.UserIdReq\x1a\x14.aioshadowsocks.User"\x00\x12@\n\nDeleteUser\x12\x19.aioshadowsocks.UserIdReq\x1a\x15.aioshadowsocks.Empty"\x00\x12?\n\x08ListUser\x12\x17.aioshadowsocks.UserReq\x1a\x18.aioshadowsocks.UserList"\x00\x12Q\n\x0bHealthCheck\x12\x1e.aioshadowsocks.HealthCheckReq\x1a\x1e.aioshadowsocks.HealthCheckRes"\x00\x30\x01\x12K\n\x0e\x46indAccessUser\x12!.aioshadowsocks.FindAccessUserReq\x1a\x14.aioshadowsocks.User"\x00\x12O\n\x0b\x44\x65\x63ryptData\x12\x1e.aioshadowsocks.DecryptDataReq\x1a\x1e.aioshadowsocks.DecryptDataRes"\x00\x62\x06proto3',
)


_USERIDREQ = _descriptor.Descriptor(
    name="UserIdReq",
    full_name="aioshadowsocks.UserIdReq",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="user_id",
            full_name="aioshadowsocks.UserIdReq.user_id",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=40,
    serialized_end=68,
)


_PORTREQ = _descriptor.Descriptor(
    name="PortReq",
    full_name="aioshadowsocks.PortReq",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="port",
            full_name="aioshadowsocks.PortReq.port",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=70,
    serialized_end=93,
)


_USERREQ = _descriptor.Descriptor(
    name="UserReq",
    full_name="aioshadowsocks.UserReq",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="user_id",
            full_name="aioshadowsocks.UserReq.user_id",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="port",
            full_name="aioshadowsocks.UserReq.port",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="method",
            full_name="aioshadowsocks.UserReq.method",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="password",
            full_name="aioshadowsocks.UserReq.password",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="enable",
            full_name="aioshadowsocks.UserReq.enable",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="tcp_conn_num",
            full_name="aioshadowsocks.UserReq.tcp_conn_num",
            index=5,
            number=6,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=95,
    serialized_end=207,
)


_HEALTHCHECKREQ = _descriptor.Descriptor(
    name="HealthCheckReq",
    full_name="aioshadowsocks.HealthCheckReq",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="url",
            full_name="aioshadowsocks.HealthCheckReq.url",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=209,
    serialized_end=238,
)


_HEALTHCHECKRES = _descriptor.Descriptor(
    name="HealthCheckRes",
    full_name="aioshadowsocks.HealthCheckRes",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="aioshadowsocks.HealthCheckRes.status_code",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="duration",
            full_name="aioshadowsocks.HealthCheckRes.duration",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=240,
    serialized_end=295,
)


_EMPTY = _descriptor.Descriptor(
    name="Empty",
    full_name="aioshadowsocks.Empty",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=297,
    serialized_end=304,
)


_USER = _descriptor.Descriptor(
    name="User",
    full_name="aioshadowsocks.User",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="user_id",
            full_name="aioshadowsocks.User.user_id",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="port",
            full_name="aioshadowsocks.User.port",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="method",
            full_name="aioshadowsocks.User.method",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="password",
            full_name="aioshadowsocks.User.password",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="enable",
            full_name="aioshadowsocks.User.enable",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="speed_limit",
            full_name="aioshadowsocks.User.speed_limit",
            index=5,
            number=6,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="access_order",
            full_name="aioshadowsocks.User.access_order",
            index=6,
            number=7,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="need_sync",
            full_name="aioshadowsocks.User.need_sync",
            index=7,
            number=8,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ip_list",
            full_name="aioshadowsocks.User.ip_list",
            index=8,
            number=9,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="tcp_conn_num",
            full_name="aioshadowsocks.User.tcp_conn_num",
            index=9,
            number=10,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="upload_traffic",
            full_name="aioshadowsocks.User.upload_traffic",
            index=10,
            number=11,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="download_traffic",
            full_name="aioshadowsocks.User.download_traffic",
            index=11,
            number=12,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=307,
    serialized_end=545,
)


_USERLIST = _descriptor.Descriptor(
    name="UserList",
    full_name="aioshadowsocks.UserList",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="data",
            full_name="aioshadowsocks.UserList.data",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=547,
    serialized_end=593,
)


_FINDACCESSUSERREQ = _descriptor.Descriptor(
    name="FindAccessUserReq",
    full_name="aioshadowsocks.FindAccessUserReq",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="port",
            full_name="aioshadowsocks.FindAccessUserReq.port",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="method",
            full_name="aioshadowsocks.FindAccessUserReq.method",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ts_protocol",
            full_name="aioshadowsocks.FindAccessUserReq.ts_protocol",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="aioshadowsocks.FindAccessUserReq.data",
            index=3,
            number=4,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=595,
    serialized_end=679,
)


_DECRYPTDATAREQ = _descriptor.Descriptor(
    name="DecryptDataReq",
    full_name="aioshadowsocks.DecryptDataReq",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="user_id",
            full_name="aioshadowsocks.DecryptDataReq.user_id",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="uuid",
            full_name="aioshadowsocks.DecryptDataReq.uuid",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="password",
            full_name="aioshadowsocks.DecryptDataReq.password",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="method",
            full_name="aioshadowsocks.DecryptDataReq.method",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="aioshadowsocks.DecryptDataReq.data",
            index=4,
            number=5,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=681,
    serialized_end=776,
)


_DECRYPTDATARES = _descriptor.Descriptor(
    name="DecryptDataRes",
    full_name="aioshadowsocks.DecryptDataRes",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="data",
            full_name="aioshadowsocks.DecryptDataRes.data",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=778,
    serialized_end=808,
)

_USERLIST.fields_by_name["data"].message_type = _USER
DESCRIPTOR.message_types_by_name["UserIdReq"] = _USERIDREQ
DESCRIPTOR.message_types_by_name["PortReq"] = _PORTREQ
DESCRIPTOR.message_types_by_name["UserReq"] = _USERREQ
DESCRIPTOR.message_types_by_name["HealthCheckReq"] = _HEALTHCHECKREQ
DESCRIPTOR.message_types_by_name["HealthCheckRes"] = _HEALTHCHECKRES
DESCRIPTOR.message_types_by_name["Empty"] = _EMPTY
DESCRIPTOR.message_types_by_name["User"] = _USER
DESCRIPTOR.message_types_by_name["UserList"] = _USERLIST
DESCRIPTOR.message_types_by_name["FindAccessUserReq"] = _FINDACCESSUSERREQ
DESCRIPTOR.message_types_by_name["DecryptDataReq"] = _DECRYPTDATAREQ
DESCRIPTOR.message_types_by_name["DecryptDataRes"] = _DECRYPTDATARES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UserIdReq = _reflection.GeneratedProtocolMessageType(
    "UserIdReq",
    (_message.Message,),
    {
        "DESCRIPTOR": _USERIDREQ,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.UserIdReq)
    },
)
_sym_db.RegisterMessage(UserIdReq)

PortReq = _reflection.GeneratedProtocolMessageType(
    "PortReq",
    (_message.Message,),
    {
        "DESCRIPTOR": _PORTREQ,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.PortReq)
    },
)
_sym_db.RegisterMessage(PortReq)

UserReq = _reflection.GeneratedProtocolMessageType(
    "UserReq",
    (_message.Message,),
    {
        "DESCRIPTOR": _USERREQ,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.UserReq)
    },
)
_sym_db.RegisterMessage(UserReq)

HealthCheckReq = _reflection.GeneratedProtocolMessageType(
    "HealthCheckReq",
    (_message.Message,),
    {
        "DESCRIPTOR": _HEALTHCHECKREQ,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.HealthCheckReq)
    },
)
_sym_db.RegisterMessage(HealthCheckReq)

HealthCheckRes = _reflection.GeneratedProtocolMessageType(
    "HealthCheckRes",
    (_message.Message,),
    {
        "DESCRIPTOR": _HEALTHCHECKRES,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.HealthCheckRes)
    },
)
_sym_db.RegisterMessage(HealthCheckRes)

Empty = _reflection.GeneratedProtocolMessageType(
    "Empty",
    (_message.Message,),
    {
        "DESCRIPTOR": _EMPTY,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.Empty)
    },
)
_sym_db.RegisterMessage(Empty)

User = _reflection.GeneratedProtocolMessageType(
    "User",
    (_message.Message,),
    {
        "DESCRIPTOR": _USER,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.User)
    },
)
_sym_db.RegisterMessage(User)

UserList = _reflection.GeneratedProtocolMessageType(
    "UserList",
    (_message.Message,),
    {
        "DESCRIPTOR": _USERLIST,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.UserList)
    },
)
_sym_db.RegisterMessage(UserList)

FindAccessUserReq = _reflection.GeneratedProtocolMessageType(
    "FindAccessUserReq",
    (_message.Message,),
    {
        "DESCRIPTOR": _FINDACCESSUSERREQ,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.FindAccessUserReq)
    },
)
_sym_db.RegisterMessage(FindAccessUserReq)

DecryptDataReq = _reflection.GeneratedProtocolMessageType(
    "DecryptDataReq",
    (_message.Message,),
    {
        "DESCRIPTOR": _DECRYPTDATAREQ,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.DecryptDataReq)
    },
)
_sym_db.RegisterMessage(DecryptDataReq)

DecryptDataRes = _reflection.GeneratedProtocolMessageType(
    "DecryptDataRes",
    (_message.Message,),
    {
        "DESCRIPTOR": _DECRYPTDATARES,
        "__module__": "aioshadowsocks_pb2"
        # @@protoc_insertion_point(class_scope:aioshadowsocks.DecryptDataRes)
    },
)
_sym_db.RegisterMessage(DecryptDataRes)


_SS = _descriptor.ServiceDescriptor(
    name="ss",
    full_name="aioshadowsocks.ss",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=811,
    serialized_end=1375,
    methods=[
        _descriptor.MethodDescriptor(
            name="CreateUser",
            full_name="aioshadowsocks.ss.CreateUser",
            index=0,
            containing_service=None,
            input_type=_USERREQ,
            output_type=_USER,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="UpdateUser",
            full_name="aioshadowsocks.ss.UpdateUser",
            index=1,
            containing_service=None,
            input_type=_USERREQ,
            output_type=_USER,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetUser",
            full_name="aioshadowsocks.ss.GetUser",
            index=2,
            containing_service=None,
            input_type=_USERIDREQ,
            output_type=_USER,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="DeleteUser",
            full_name="aioshadowsocks.ss.DeleteUser",
            index=3,
            containing_service=None,
            input_type=_USERIDREQ,
            output_type=_EMPTY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="ListUser",
            full_name="aioshadowsocks.ss.ListUser",
            index=4,
            containing_service=None,
            input_type=_USERREQ,
            output_type=_USERLIST,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="HealthCheck",
            full_name="aioshadowsocks.ss.HealthCheck",
            index=5,
            containing_service=None,
            input_type=_HEALTHCHECKREQ,
            output_type=_HEALTHCHECKRES,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="FindAccessUser",
            full_name="aioshadowsocks.ss.FindAccessUser",
            index=6,
            containing_service=None,
            input_type=_FINDACCESSUSERREQ,
            output_type=_USER,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="DecryptData",
            full_name="aioshadowsocks.ss.DecryptData",
            index=7,
            containing_service=None,
            input_type=_DECRYPTDATAREQ,
            output_type=_DECRYPTDATARES,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_SS)

DESCRIPTOR.services_by_name["ss"] = _SS

# @@protoc_insertion_point(module_scope)



from shadowsocks.ciphers import SUPPORT_METHODS
from shadowsocks.gen.async_protos.aioshadowsocks_grpc import ssBase
from shadowsocks.gen.async_protos.aioshadowsocks_pb2 import (
    DecryptDataRes,
    Empty,
    HealthCheckRes,
    User,
    UserList,
)
from shadowsocks.mdb import models as m


class AioShadowsocksServicer(ssBase):
    def __init__(self) -> None:
        self.cipher_map = {}

    async def CreateUser(self, stream):
        request = await stream.recv_message()
        data = {
            "user_id": request.user_id,
            "port": request.port,
            "method": request.method,
            "password": request.password,
            "enable": request.enable,
        }
        user = m.User.create_or_update_user_from_data(data)
        await stream.send_message(User(**user.to_dict()))

    async def UpdateUser(self, stream):
        request = await stream.recv_message()
        data = {
            "user_id": request.user_id,
            "port": request.port,
            "method": request.method,
            "password": request.password,
            "enable": request.enable,
        }
        user = m.User.create_or_update_user_from_data(data)
        await stream.send_message(User(**user.to_dict()))

    async def GetUser(self, stream):
        request = await stream.recv_message()
        user = m.User.get_by_id(request.user_id)
        await stream.send_message(User(**user.to_dict()))

    async def DeleteUser(self, stream):
        request = await stream.recv_message()
        user = m.User.get_by_id(request.user_id)
        user.server.close_server()
        user.delete_instance()
        await stream.send_message(Empty())

    async def ListUser(self, stream):
        request = await stream.recv_message()
        users = m.User.select().where(m.User.tcp_conn_num <= request.tcp_conn_num)
        res = UserList(data=[user.to_dict() for user in users])
        await stream.send_message(res)

    async def HealthCheck(self, stream):
        request = await stream.recv_message()
        url = request.url
        await stream.send_message(HealthCheckRes(status_code="200", duration=100))

    async def FindAccessUser(self, stream):
        request = await stream.recv_message()
        user = m.User.find_access_user(
            request.port, request.method, request.ts_protocol, request.data
        )
        if not user:
            raise Exception("not find")
        await stream.send_message(User(**user.to_dict()))

    async def DecryptData(self, stream):
        request = await stream.recv_message()
        cipher = self.cipher_map.get(request.uuid)
        if not cipher:
            cipher = SUPPORT_METHODS[request.method](request.password)
            self.cipher_map[request.uuid] = cipher
        await stream.send_message(DecryptDataRes(data=cipher.decrypt(request.data)))



import grpc

from shadowsocks.gen.sync_protos.aioshadowsocks_pb2 import (
    DecryptDataReq,
    FindAccessUserReq,
    User,
    UserIdReq,
)
from shadowsocks.gen.sync_protos.aioshadowsocks_pb2_grpc import ssStub


class SSClient:
    def __init__(self, rpc_endpint="127.0.0.1:5000") -> None:
        self.channel = grpc.insecure_channel(rpc_endpint)
        self.stub = ssStub(self.channel)

    def get_user(self, user_id: int) -> User:
        user = self.stub.GetUser(UserIdReq(user_id=user_id))
        print(f"user: {user}")
        return user

    def find_access_user(self, port, method, ts_protocol, data):
        req = FindAccessUserReq(
            port=port, method=method, ts_protocol=ts_protocol, data=data
        )
        return self.stub.FindAccessUser(req)

    def decrypt_data(self, uuid, user_id, method, password, data):
        req = DecryptDataReq(
            uuid=uuid, user_id=user_id, method=method, password=password, data=data
        )
        return self.stub.DecryptData(req).data



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: aioshadowsocks.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.client
import grpclib.const

if typing.TYPE_CHECKING:
    import grpclib.server

from . import aioshadowsocks_pb2


class ssBase(abc.ABC):
    @abc.abstractmethod
    async def CreateUser(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.UserReq, aioshadowsocks_pb2.User]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def UpdateUser(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.UserReq, aioshadowsocks_pb2.User]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def GetUser(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.UserIdReq, aioshadowsocks_pb2.User]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def DeleteUser(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.UserIdReq, aioshadowsocks_pb2.Empty]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def ListUser(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.UserReq, aioshadowsocks_pb2.UserList]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def HealthCheck(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.HealthCheckReq, aioshadowsocks_pb2.HealthCheckRes]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def FindAccessUser(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.FindAccessUserReq, aioshadowsocks_pb2.User]",
    ) -> None:
        pass

    @abc.abstractmethod
    async def DecryptData(
        self,
        stream: "grpclib.server.Stream[aioshadowsocks_pb2.DecryptDataReq, aioshadowsocks_pb2.DecryptDataRes]",
    ) -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            "/aioshadowsocks.ss/CreateUser": grpclib.const.Handler(
                self.CreateUser,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.UserReq,
                aioshadowsocks_pb2.User,
            ),
            "/aioshadowsocks.ss/UpdateUser": grpclib.const.Handler(
                self.UpdateUser,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.UserReq,
                aioshadowsocks_pb2.User,
            ),
            "/aioshadowsocks.ss/GetUser": grpclib.const.Handler(
                self.GetUser,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.UserIdReq,
                aioshadowsocks_pb2.User,
            ),
            "/aioshadowsocks.ss/DeleteUser": grpclib.const.Handler(
                self.DeleteUser,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.UserIdReq,
                aioshadowsocks_pb2.Empty,
            ),
            "/aioshadowsocks.ss/ListUser": grpclib.const.Handler(
                self.ListUser,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.UserReq,
                aioshadowsocks_pb2.UserList,
            ),
            "/aioshadowsocks.ss/HealthCheck": grpclib.const.Handler(
                self.HealthCheck,
                grpclib.const.Cardinality.UNARY_STREAM,
                aioshadowsocks_pb2.HealthCheckReq,
                aioshadowsocks_pb2.HealthCheckRes,
            ),
            "/aioshadowsocks.ss/FindAccessUser": grpclib.const.Handler(
                self.FindAccessUser,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.FindAccessUserReq,
                aioshadowsocks_pb2.User,
            ),
            "/aioshadowsocks.ss/DecryptData": grpclib.const.Handler(
                self.DecryptData,
                grpclib.const.Cardinality.UNARY_UNARY,
                aioshadowsocks_pb2.DecryptDataReq,
                aioshadowsocks_pb2.DecryptDataRes,
            ),
        }


class ssStub:
    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.CreateUser = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/CreateUser",
            aioshadowsocks_pb2.UserReq,
            aioshadowsocks_pb2.User,
        )
        self.UpdateUser = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/UpdateUser",
            aioshadowsocks_pb2.UserReq,
            aioshadowsocks_pb2.User,
        )
        self.GetUser = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/GetUser",
            aioshadowsocks_pb2.UserIdReq,
            aioshadowsocks_pb2.User,
        )
        self.DeleteUser = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/DeleteUser",
            aioshadowsocks_pb2.UserIdReq,
            aioshadowsocks_pb2.Empty,
        )
        self.ListUser = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/ListUser",
            aioshadowsocks_pb2.UserReq,
            aioshadowsocks_pb2.UserList,
        )
        self.HealthCheck = grpclib.client.UnaryStreamMethod(
            channel,
            "/aioshadowsocks.ss/HealthCheck",
            aioshadowsocks_pb2.HealthCheckReq,
            aioshadowsocks_pb2.HealthCheckRes,
        )
        self.FindAccessUser = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/FindAccessUser",
            aioshadowsocks_pb2.FindAccessUserReq,
            aioshadowsocks_pb2.User,
        )
        self.DecryptData = grpclib.client.UnaryUnaryMethod(
            channel,
            "/aioshadowsocks.ss/DecryptData",
            aioshadowsocks_pb2.DecryptDataReq,
            aioshadowsocks_pb2.DecryptDataRes,
        )



import socket

from prometheus_client import Counter, Gauge, Histogram, Info

# METRICS
NODE_HOST_NAME = socket.gethostname()

SS_NODE_INFO = Info("ss_node", "ss node info")
SS_NODE_INFO.info({"ss_node_name": NODE_HOST_NAME})

CONNECTION_MADE_COUNT = Counter(
    "connection_made_count",
    "shadowsocks connection made number",
    labelnames=[
        "ss_node",
    ],
)
CONNECTION_MADE_COUNT = CONNECTION_MADE_COUNT.labels(ss_node=NODE_HOST_NAME)


ACTIVE_CONNECTION_COUNT = Gauge(
    "active_connection_count",
    "shadowsocks active connection count",
    labelnames=[
        "ss_node",
    ],
)
ACTIVE_CONNECTION_COUNT = ACTIVE_CONNECTION_COUNT.labels(ss_node=NODE_HOST_NAME)


NETWORK_TRANSMIT_BYTES = Counter(
    "network_transmit_bytes",
    "shadowsocks network transmit bytes",
    labelnames=[
        "ss_node",
    ],
)
NETWORK_TRANSMIT_BYTES = NETWORK_TRANSMIT_BYTES.labels(ss_node=NODE_HOST_NAME)


ENCRYPT_DATA_TIME = Histogram(
    "encrypt_data_time_seconds",
    "shadowsocks encrypt data time seconds",
    labelnames=[
        "ss_node",
    ],
)
ENCRYPT_DATA_TIME = ENCRYPT_DATA_TIME.labels(ss_node=NODE_HOST_NAME)


DECRYPT_DATA_TIME = Histogram(
    "decrypt_data_time_seconds",
    "shadowsocks decrypt data time seconds",
    labelnames=[
        "ss_node",
    ],
)
DECRYPT_DATA_TIME = DECRYPT_DATA_TIME.labels(ss_node=NODE_HOST_NAME)


FIND_ACCESS_USER_TIME = Histogram(
    "find_access_user_time_seconds",
    "time to find access user",
    labelnames=[
        "ss_node",
    ],
)
FIND_ACCESS_USER_TIME = FIND_ACCESS_USER_TIME.labels(ss_node=NODE_HOST_NAME)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shadowsocks',
 'shadowsocks.gen.async_protos',
 'shadowsocks.gen.sync_protos',
 'shadowsocks.mdb']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp==3.6.2',
 'bloom-filter==1.3',
 'cryptography>=3.2.1,<4.0.0',
 'fire==0.3.1',
 'grpclib==0.4.1',
 'hkdf==0.0.3',
 'httpx>=0.16.1,<0.17.0',
 'peewee==3.14.0',
 'prometheus-async==19.2.0',
 'prometheus-client==0.8.0',
 'protobuf==3.13.0',
 'sentry-sdk==0.18.0']

entry_points = \
{'console_scripts': ['aioss = shadowsocks.__main__:main']}

setup_kwargs = {
    'name': 'aioshadowsocks',
    'version': '0.1.5',
    'description': 'shadowsocks build with asyncio , also support many user in one port',
    'long_description': '# aioshadowsocks\n\n用 asyncio 重写 shadowsocks\n\n![Publish Docker](https://github.com/Ehco1996/aioshadowsocks/workflows/Publish%20Docker/badge.svg?branch=master)\n\n## 视频安装教程\n\n* 面板视频安装教程: [地址](https://youtu.be/BRHcdGeufvY)\n\n* 后端对接视频教程: [地址](https://youtu.be/QNbnya1HHU0)\n\n* 隧道对接视频教程: [地址](https://youtu.be/R4U0NZaMUeY)\n\n## 使用\n\n* 安装\n\n``` sh\npip install aioshadowsocks\n```\n\n* 多用户配置\n\naioshadowsocks 将json作为配置文件, 会读取当前目录下 `userconfigs.json` 作为默认的配置文件\n\n``` json\n{\n    "users": [\n        {\n            "user_id": 1,\n            "port": 2345,\n            "method": "none",\n            "password": "hellotheworld1",\n            "transfer": 104857600,\n            "speed_limit": 0\n        },\n        {\n            "user_id": 2,\n            "port": 2346,\n            "method": "chacha20-ietf-poly1305",\n            "password": "hellotheworld2",\n            "transfer": 104857600,\n            "speed_limit": 384000\n        }\n    ]\n}\n```\n\n同时也支持从http服务器读取配置文件，这时需要注入环境变量 `SS_API_ENDPOINT` 作为读取配置的api地址\n\n* 注入环境变量\n\n `export SS_API_ENDPOINT="https://xxx/com"`\n\n* 启动ss服务器\n\n``` bash\naioss run_ss_server\n```\n\n## Docker Version\n\n1. install docker\n\n``` sh\ncurl -sSL https://get.docker.com/ | sh\n```\n\n2. install docker-compose\n\n``` sh\nsudo curl -L "https://github.com/docker/compose/releases/download/1.23.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/local/bin/docker-compose\n```\n\n3. apply executable permissions\n\n``` sh\nsudo chmod +x /usr/local/bin/docker-compose\n```\n\n4. run server\n\n``` sh\ndocker-compose up\n```\n\n## 为什么要重写shadowsocks\n\n主要想通过这个项目的推进来深入了解 `asyncio`\n\n另外我的一个项目: [django-sspanel](https://github.com/Ehco1996/django-sspanel) 依赖 `shadowsocksr`\n\n但该项目已经停止开发了，所以决定重新造个轮子\n\n## 主要功能\n\n* tcp/udp 代理\n* 流量统计\n* 速率控制\n* 开放了grpc接口(类似ss-manager)\n* **单端口多用户（利用AEAD加密在不破坏协议的情况下实现）**\n* **prometheus/grafana metrics监控** （dashboard在项目的static/grafana/文件夹下）\n\n## 监控dashboard\n\n![](static/images/1.png)\n\n![](statoc/images/2.png)\n\n![](static/images/3.png)\n\n## 性能测试\n\n> Shadowsocks本身是一个IO密集行的应用，但是由于加入了AEAD加密，使得SS本身变成了CPU密集行的应用\n> 而Python本身是不太适合CPU密集的场景的，所以在AEAD模式中的表现不佳\n> PS: 当然，其实是我代码写的烂，python不背锅\n\n* Steam-Cipher-None(不加密 高IO)\n\n![](static/images/stream-none.png)\n\n* AEAD-Cipher-CHACHA-20(加密 高CPU)\n\n![](static/images/aead-chacha-20-ietf-poly-1305.png)\n\n## rpc proto\n\n``` protobuf\nsyntax = "proto3";\n\npackage aioshadowsocks;\n\n// REQ\nmessage UserIdReq { int32 user_id = 1; }\nmessage PortReq { int32 port = 1; }\n\nmessage UserReq {\n  int32 user_id = 1;\n  int32 port = 2;\n  string method = 3;\n  string password = 4;\n  bool enable = 5;\n}\n\n// RES\nmessage Empty {}\n\nmessage User {\n  int32 user_id = 1;\n  int32 port = 2;\n  string method = 3;\n  string password = 4;\n  bool enable = 5;\n  int32 speed_limit = 6;\n  int32 access_order = 7;\n  bool need_sync = 8;\n  repeated string ip_list = 9;\n  int32 tcp_conn_num = 10;\n  int64 upload_traffic = 11;\n  int64 download_traffic = 12;\n}\n\n// service\nservice ss {\n  rpc CreateUser(UserReq) returns (User) {}\n  rpc UpdateUser(UserReq) returns (User) {}\n  rpc GetUser(UserIdReq) returns (User) {}\n  rpc DeleteUser(UserIdReq) returns (Empty) {}\n}\n```\n',
    'author': 'ehco1996',
    'author_email': 'zh19960202@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0,<3.9.0',
}


setup(**setup_kwargs)



import asyncio
import json
import logging
from collections import defaultdict

import httpx

from shadowsocks.core import LocalTCP, LocalUDP
from shadowsocks.mdb.models import User, db


class ProxyMan:
    """
    1. 将model的调用都放在这里
    2. server相关的东西也放在这里
    app -> proxyman -> core ->cipherman/model
    """

    AEAD_METHOD_LIST = [
        "chacha20-ietf-poly1305",
        "aes-128-gcm",
        "aes-256-gcm",
    ]

    def __init__(self, use_json, sync_time, listen_host, api_endpoint):
        self.use_json = use_json
        self.sync_time = sync_time
        self.listen_host = listen_host
        self.api_endpoint = api_endpoint
        self.loop = asyncio.get_event_loop()
        # NOTE {"port":{"tcp":tcp_server,"udp":udp_server}}
        self.__running_servers__ = defaultdict(dict)

    @staticmethod
    def create_or_update_from_json(path):
        with open(path, "r") as f:
            data = json.load(f)
            User.create_or_update_by_user_data_list(data["users"])

    @staticmethod
    async def get_user_from_remote(url):
        async with httpx.AsyncClient() as client:
            res = await client.get(url)
            User.create_or_update_by_user_data_list(res.json()["users"])

    @staticmethod
    async def flush_metrics_to_remote(url):
        fields = [
            User.user_id,
            User.ip_list,
            User.tcp_conn_num,
            User.upload_traffic,
            User.download_traffic,
        ]
        with db.atomic("EXCLUSIVE"):
            users = list(User.select(*fields).where(User.need_sync == True))
            User.update(
                ip_list=set(), upload_traffic=0, download_traffic=0, need_sync=False
            ).where(User.need_sync == True).execute()

        data = []
        for user in users:
            data.append(
                {
                    "user_id": user.user_id,
                    "ip_list": list(user.ip_list),
                    "tcp_conn_num": user.tcp_conn_num,
                    "upload_traffic": user.upload_traffic,
                    "download_traffic": user.download_traffic,
                }
            )
        async with httpx.AsyncClient() as client:
            await client.post(url, json={"data": data})

    async def sync_from_remote_cron(self):
        try:
            await self.flush_metrics_to_remote(self.api_endpoint)
            await self.get_user_from_remote(self.api_endpoint)
        except Exception as e:
            logging.warning(f"sync user from remote error {e}")

    @classmethod
    async def sync_from_json_cron(self, sync_time):
        try:
            User.create_or_update_from_json("userconfigs.json")
        except Exception as e:
            logging.warning(f"sync user from json error {e}")

    def get_server_by_port(self, port):
        return self.__running_servers__.get(port)

    async def start_and_check_ss_server(self):
        """
        启动ss server并且定期检查是否要开启新的server
        TODO 关闭不需要的server
        """

        if self.use_json:
            await self.sync_from_json_cron()
        else:
            await self.sync_from_remote_cron()

        for user in User.select().where(User.enable == True):
            try:
                await self.init_server(user)
            except Exception as e:
                logging.error(e)
                self.loop.stop()
        self.loop.call_later(
            self.sync_time,
            self.loop.create_task,
            self.start_and_check_ss_server(),
        )

    async def init_server(self, user: User):

        running_server = self.get_server_by_port(user.port)
        if running_server:
            return

        tcp_server = await self.loop.create_server(
            LocalTCP(user.port), self.listen_host, user.port, reuse_port=True
        )
        udp_server, _ = await self.loop.create_datagram_endpoint(
            LocalUDP(user.port), (self.listen_host, user.port), reuse_port=True
        )
        self.__running_servers__[user.port] = {
            "tcp": tcp_server,
            "udp": udp_server,
        }
        logging.info(
            "user:{} method:{} password:{} {}:{} 已启动".format(
                user, user.method, user.password, self.listen_host, user.port
            )
        )

    def close_server(self):
        for port, server_data in self.__running_servers__.items():
            server_data["tcp"].close()
            server_data["udp"].close()
            logging.info(f"port:{port} 已关闭!")
